@@ -5,11 +5,11 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2017, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2020, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
- * are also available at https://curl.haxx.se/docs/copyright.html.
+ * are also available at https://curl.se/docs/copyright.html.
  *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
@@ -35,7 +35,7 @@ int main(void)
   CURL *curl;
 
   CURLM *multi_handle;
-  int still_running;
+  int still_running = 0;
 
   curl_mime *form = NULL;
   curl_mimepart *field = NULL;
@@ -69,7 +69,7 @@ int main(void)
     headerlist = curl_slist_append(headerlist, buf);
 
     /* what URL that receives this POST */
-    curl_easy_setopt(curl, CURLOPT_URL, "http://www.example.com/upload.cgi");
+    curl_easy_setopt(curl, CURLOPT_URL, "https://www.example.com/upload.cgi");
     curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
 
     curl_easy_setopt(curl, CURLOPT_HTTPHEADER, headerlist);
@@ -79,7 +79,7 @@ int main(void)
 
     curl_multi_perform(multi_handle, &still_running);
 
-    do {
+    while(still_running) {
       struct timeval timeout;
       int rc; /* select() return code */
       CURLMcode mc; /* curl_multi_fdset() return code */
@@ -150,7 +150,7 @@ int main(void)
         printf("running: %d!\n", still_running);
         break;
       }
-    } while(still_running);
+    }
 
     curl_multi_cleanup(multi_handle);
 
