@@ -7,11 +7,11 @@
  *                            | (__| |_| |  _ <| |___
  *                             \___|\___/|_| \_\_____|
  *
- * Copyright (C) 1998 - 2013, Daniel Stenberg, <daniel@haxx.se>, et al.
+ * Copyright (C) 1998 - 2020, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution. The terms
- * are also available at http://curl.haxx.se/docs/copyright.html.
+ * are also available at https://curl.se/docs/copyright.html.
  *
  * You may opt to use, copy, modify, merge, publish, distribute and/or sell
  * copies of the Software, and permit persons to whom the Software is
@@ -22,61 +22,59 @@
  *
  ***************************************************************************/
 #include "tool_setup.h"
+#include "tool_sdecls.h"
+
+struct GlobalConfig;
+struct OperationConfig;
 
 /* returns 1 for success, 0 otherwise (we use OpenSSL *_Init fncs directly) */
-typedef int (* Curl_digest_init_func)(void *context);
+typedef int (*digest_init_func)(void *context);
 
-typedef void (* Curl_digest_update_func)(void *context,
-                                         const unsigned char *data,
-                                         unsigned int len);
-typedef void (* Curl_digest_final_func)(unsigned char *result, void *context);
+typedef void (*digest_update_func)(void *context,
+                                   const unsigned char *data,
+                                   unsigned int len);
+typedef void (*digest_final_func)(unsigned char *result, void *context);
 
-typedef struct {
-  Curl_digest_init_func     digest_init;   /* Initialize context procedure */
-  Curl_digest_update_func   digest_update; /* Update context with data */
-  Curl_digest_final_func    digest_final;  /* Get final result procedure */
-  unsigned int           digest_ctxtsize;  /* Context structure size */
-  unsigned int           digest_resultlen; /* Result length (bytes) */
-} digest_params;
+struct digest_params {
+  digest_init_func     digest_init;   /* Initialize context procedure */
+  digest_update_func   digest_update; /* Update context with data */
+  digest_final_func    digest_final;  /* Get final result procedure */
+  unsigned int         digest_ctxtsize;  /* Context structure size */
+  unsigned int         digest_resultlen; /* Result length (bytes) */
+};
 
-typedef struct {
-  const digest_params   *digest_hash;      /* Hash function definition */
+struct digest_context {
+  const struct digest_params *digest_hash; /* Hash function definition */
   void                  *digest_hashctx;   /* Hash function context */
-} digest_context;
+};
 
-digest_context * Curl_digest_init(const digest_params *dparams);
-int Curl_digest_update(digest_context *context,
-                       const unsigned char *data,
-                       unsigned int len);
-int Curl_digest_final(digest_context *context, unsigned char *result);
-
-typedef struct {
+struct metalink_digest_def {
   const char *hash_name;
-  const digest_params *dparams;
-} metalink_digest_def;
+  const struct digest_params *dparams;
+};
 
-typedef struct {
+struct metalink_digest_alias {
   const char *alias_name;
-  const metalink_digest_def *digest_def;
-} metalink_digest_alias;
+  const struct metalink_digest_def *digest_def;
+};
 
-typedef struct metalink_checksum {
-  const metalink_digest_def *digest_def;
+struct metalink_checksum {
+  const struct metalink_digest_def *digest_def;
   /* raw digest value, not ascii hex digest */
   unsigned char *digest;
-} metalink_checksum;
+};
 
-typedef struct metalink_resource {
+struct metalink_resource {
   struct metalink_resource *next;
   char *url;
-} metalink_resource;
+};
 
-typedef struct metalinkfile {
+struct metalinkfile {
   struct metalinkfile *next;
   char *filename;
-  metalink_checksum *checksum;
-  metalink_resource *resource;
-} metalinkfile;
+  struct metalink_checksum *checksum;
+  struct metalink_resource *resource;
+};
 
 #ifdef USE_METALINK
 
@@ -91,17 +89,19 @@ typedef struct metalinkfile {
                                     (CURL_REQ_LIBMETALINK_MINOR * 100) + \
                                      CURL_REQ_LIBMETALINK_PATCH)
 
-extern const digest_params MD5_DIGEST_PARAMS[1];
-extern const digest_params SHA1_DIGEST_PARAMS[1];
-extern const digest_params SHA256_DIGEST_PARAMS[1];
+extern const struct digest_params MD5_DIGEST_PARAMS[1];
+extern const struct digest_params SHA1_DIGEST_PARAMS[1];
+extern const struct digest_params SHA256_DIGEST_PARAMS[1];
 
 #include <metalink/metalink.h>
 
 /*
  * Counts the resource in the metalinkfile.
  */
-int count_next_metalink_resource(metalinkfile *mlfile);
-void clean_metalink(struct Configurable *config);
+int count_next_metalink_resource(struct metalinkfile *mlfile);
+
+void delete_metalinkfile(struct metalinkfile *mlfile);
+void clean_metalink(struct OperationConfig *config);
 
 /*
  * Performs final parse operation and extracts information from
@@ -113,7 +113,7 @@ void clean_metalink(struct Configurable *config);
  * -1: Parsing failed; or no file is found
  * -2: Parsing succeeded with some warnings.
  */
-int parse_metalink(struct Configurable *config, struct OutStruct *outs,
+int parse_metalink(struct OperationConfig *config, struct OutStruct *outs,
                    const char *metalink_url);
 
 /*
@@ -142,8 +142,8 @@ int check_metalink_content_type(const char *content_type);
  *   No checksum in Metalink supported, hash algorithm not available, or
  *   Metalink does not contain checksum.
  */
-int metalink_check_hash(struct Configurable *config,
-                        metalinkfile *mlfile,
+int metalink_check_hash(struct GlobalConfig *config,
+                        struct metalinkfile *mlfile,
                         const char *filename);
 
 /*
@@ -154,7 +154,8 @@ void metalink_cleanup(void);
 #else /* USE_METALINK */
 
 #define count_next_metalink_resource(x)  0
-#define clean_metalink(x)  Curl_nop_stmt
+#define delete_metalinkfile(x)  (void)x
+#define clean_metalink(x)  (void)x
 
 /* metalink_cleanup() takes no arguments */
 #define metalink_cleanup() Curl_nop_stmt
